/** Copyright (C) 2010 Maxime Gamboni. See COPYING for
    copying/warranty info. */

#include <math.h>
#include <SDL/SDL_audio.h>

#include "main.h"
#include "level.h"
#include "server.h"

//GLuint tex_id;

GLuint extra_tex;
GLuint fire_tex;
GLuint wide_tex;

int VideoFlags = 0;                 // Video Flags for the Create Window function
SDL_Surface * mainWindow = NULL;    // drawing surface on the SDL window

int screen_width, screen_height;
double screen_ratio;

//Stone *stones[DCOUNT];
//Cell cells[SWID][SHEI];

//Pad pad = Pad();
//Ball ball = Ball();

Sound click[4];
Sound crack[4];
Sound *tock = new Sound();

Level *level;
//int levelnum=1;
//bool levelDone=false;

SDL_Joystick *joy=0;
double joyy;
double joyx;

Server *connection=0;

Uint32 triggerRefresh(Uint32 interval,void *param) {
  static SDL_Event ev;

  /* just send a dummy event */
  ev.type=SDL_USEREVENT;
  ev.user.code=2;
  ev.user.data1=NULL;
  ev.user.data2=NULL;

  SDL_PushEvent(&ev);
  return interval;
}

/* Usage: $0 [-w] [-s] [-c <ip>] [levelname] where -w opens in a
   Window instead of full screen, -s activates Server mode and -c
   followed by an ip address activates Client mode. */

bool init( int argc,char *argv[] ) {
  const char *lvl = 0;

  bool fullscreen = true;

  bool client=false;

  screen_width = 0; // default: zero makes full screen
  screen_height = 0;


  for (int i=1;i<argc;i++) {
    if (!strcmp(argv[i],"-w")) {
      screen_width = 700;
      screen_height = 400;

      fullscreen = false;
    } else if (!strcmp(argv[i],"-s")) {
      connection = new Server();
      connection->server();
    } else if (!strcmp(argv[i],"-c")) {
      connection = new Server();
      if (argc>i+1) {
	if (connection->client(argv[++i])) 
	  client=true;
	else
	  return false;
      } else {
	printf("Warning, -c is missing a parameter. Using localhost.\n");
	if (connection->client("127.0.0.1"))
	  client=true;
	else
	  return false;
      }
    } else {
      lvl = argv[i];
    }
  }

  srand(time(0));

  if( SDL_Init( SDL_INIT_VIDEO | SDL_INIT_TIMER | SDL_INIT_JOYSTICK ) < 0 )                // try to initialize SDL video module
    {
        cerr << "Failed initializing SDL Video : " << SDL_GetError() << endl;      // report error if it fails
    }

  VideoFlags    = SDL_OPENGL|SDL_HWPALETTE;

    const SDL_VideoInfo * VideoInfo = SDL_GetVideoInfo();     // query SDL for information about our video hardware

    if(VideoInfo == NULL)                                     // if this query fails
    {
        cerr << "Failed getting Video Info : " << SDL_GetError() << endl;          // report error
        exit(1);
    } 

    ///////////// we set the system dependant flags here
    if(VideoInfo -> hw_available)                      // is it a hardware surface
        VideoFlags |= SDL_HWSURFACE;
    else
        VideoFlags |= SDL_SWSURFACE;

    // Blitting is fast copying / moving /swapping of contiguous sections of memory
    // for more about blitting check out : http://www.csc.liv.ac.uk/~fish/HTML/blitzman/bm_blitter.html
    if(VideoInfo -> blit_hw)                            // is hardware blitting available
        VideoFlags |= SDL_HWACCEL;
 
    if (fullscreen) {
      VideoFlags |= SDL_FULLSCREEN;
    } else {
      VideoFlags |= SDL_NOFRAME;
    }

    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );      // tell SDL that the GL drawing is going to be double buffered
    SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE,   SCREEN_DEPTH);         // size of depth buffer 
    SDL_GL_SetAttribute( SDL_GL_STENCIL_SIZE, 0);       // we aren't going to use the stencil buffer
    SDL_GL_SetAttribute( SDL_GL_ACCUM_RED_SIZE, 0);     // this and the next three lines set the bits allocated per pixel -
    SDL_GL_SetAttribute( SDL_GL_ACCUM_GREEN_SIZE, 0);   // - for the accumulation buffer to 0
    SDL_GL_SetAttribute( SDL_GL_ACCUM_BLUE_SIZE, 0);
    SDL_GL_SetAttribute( SDL_GL_ACCUM_ALPHA_SIZE, 0);

    mainWindow = SDL_SetVideoMode(screen_width,screen_height, SCREEN_DEPTH, VideoFlags);     // SCREEN_DEPTH is macro for bits per pixel

    if( mainWindow == NULL )                            // if window creation failed
    {
        cerr << "Failed to Create Window : " << SDL_GetError() << endl;         // report error 
        exit(2);
    } 

    // const SDL_Surface * surf = SDL_GetVideoSurface();

    screen_width = mainWindow->w;
    screen_height = mainWindow->h;
    /* Note that PFOV = 2*atan(1/2) */
    screen_ratio = atan(((double)screen_width)/screen_height/2) / (PFOV/2);

    //    printf("screenratio = %f\n",screen_ratio);

    SDL_ShowCursor(SDL_DISABLE);

    SDL_AddTimer(20,&triggerRefresh,0);

  // gouraud mode
  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
  glShadeModel(GL_SMOOTH);

  /*  glHint( GL_POINT_SMOOTH_HINT, GL_NICEST );
  glHint( GL_LINE_SMOOTH_HINT, GL_NICEST );
  glHint( GL_POLYGON_SMOOTH_HINT, GL_NICEST );
  glHint( GL_FOG_HINT, GL_NICEST );
  glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );
  */

  glEnable(GL_LIGHTING);
  glColorMaterial ( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE );
  glEnable ( GL_COLOR_MATERIAL );
  glEnable ( GL_LIGHT0);
  GLfloat white[] = {1.0f, 1.0f, 1.0f , 1.0f};
  glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, white);
  glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS,25);

  glEnable (GL_BLEND); glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glAlphaFunc ( GL_GREATER, 0.1 );  glEnable ( GL_ALPHA_TEST );

  glEnable(GL_TEXTURE_2D);
 
  // How Image data are stored in the GPU's memory    
  glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );   

  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT ); 
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
    
  //glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, tex->env_mode );  
    
  if (!loadTexture("images/extra.jpeg",&extra_tex)) {
    printf("couldn't load texture extra.jpeg\n");
    return false;
  }
  if (!loadTexture("images/fire.jpeg",&fire_tex)) {
    printf("couldn't load texture fire.jpeg\n");
    return false;
  }
  if (!loadTexture("images/wide.jpeg",&wide_tex)) {
    printf("couldn't load texture wide.jpeg\n");
    return false;
  }

    glClearDepth( 1.0f ); 
    glEnable( GL_DEPTH_TEST );
        
    glMatrixMode(GL_PROJECTION);                        // Select The Projection Matrix
    glLoadIdentity();                                   // Reset The Projection Matrix
 
                  // FOV        // Width Height Ratio   //  The farthest distance before it stops drawing)
    //gluPerspective( 45.0f, (GLdouble)width/(GLdouble)height, 1.0f, 10000.0f );
    glPerspective( 45.0, (GLdouble)screen_width/(GLdouble)screen_height, 1.0, 10000.0 );
 
    glMatrixMode(GL_MODELVIEW);                         // Select The Modelview Matrix
    glLoadIdentity();                                   // Reset The Modelview Matrix
  //    Clock::instance()->startFrame();

    /** Audio initialisation from http://www.libsdl.org/intro.en/usingsound.html **/

    SDL_AudioSpec fmt;

    /* Set 16-bit stereo audio at 22Khz */
    fmt.freq = 22050;
    fmt.format = AUDIO_S16;
    fmt.channels = 2;
    fmt.samples = 512;        /* A good value for games */
    fmt.callback = mixaudio; // that's in sound.h
    fmt.userdata = NULL;

    /* Open the audio device and start playing sound! */
    if ( SDL_OpenAudio(&fmt, NULL) < 0 ) {
      fprintf(stderr, "Unable to open audio: %s\n", SDL_GetError());
      return false;
    }
    SDL_PauseAudio(0);

    for (int i=0;i<4;i++) {
      char fn[20];
      snprintf(fn,20,"sounds/click%d.wav",i);
      click[i] = Sound();
      click[i].loadWAV(fn);
    }

    for (int i=0;i<4;i++) {
      char fn[20];
      snprintf(fn,20,"sounds/crack%d.wav",i);
      crack[i] = Sound();
      crack[i].loadWAV(fn);
    }
    tock->loadWAV("sounds/tock.wav");

    /* Experimental joystick-related code */
    joyx=0;
    joyy=0;
    // Check for joystick
    if(SDL_NumJoysticks()>0){
      // Open joystick
      joy=SDL_JoystickOpen(0);

      if(!joy) printf("Couldn't open Joystick 0\n");
      // else
      // 	{
      // 	  printf("Opened Joystick 0\n");
      // 	  printf("Name: %s\n", SDL_JoystickName(0));
      // 	  printf("Number of Axes: %d\n", SDL_JoystickNumAxes(joy));
      // 	  printf("Number of Buttons: %d\n", SDL_JoystickNumButtons(joy));
      // 	  printf("Number of Balls: %d\n", SDL_JoystickNumBalls(joy));
      // 	}

    // } else {
    //   printf("No joystick found\n");
    }
    
    level = new Level(client);

    return client || level->load(lvl);
}
