#ifndef LITTLECUBE_H
#define LITTLECUBE_H

/** A small object that can move around the level, that bounces on
    walls and stone. For instance: the ball, what the pad fires, and
    what you get when a stone breaks.

    Copyright (C) 2010 Maxime Gamboni. See COPYING for
    copying/warranty info. */


//class Littlecube;

class Level; //#include "level.h"

#include "item.h"

class Littlecube : public Item {
 public:
  Littlecube(unsigned int id);
  ~Littlecube();

  /* The triggers parameter will be passed to check_collision, i.e. if
     true, little cubes trigger other stones. gravity: whether they're
     affected by gravity. r: the radius. */
  void init(double x,double y,double r,bool triggers,bool gravity=true);

  /* Set the position, leaving speed intact. */
  void move(double x,double y);

  /* If you set the speed to exactly zero then colision detection is
     deactivated */
  void set_speed(double vx,double vy);

  /* The radius will increase by dr every second (typically dr is
     negative). For dr<0, radius stops decreasing when it reaches
     zero. */
  void set_delta_radius(double dr);

  /* Returns true if the radius reached zero */
  bool has_vanished();

  /* Returns true if there was a collision (We need the level to do
     collision detection) */
  bool update(double t,Level *lvl);

  /* Make sure this is visible on the screen. See lookAt in
     main.h. The request is transient if delta_radius is non-zero, in
     particular when the ball is leaving the level area. */
  void look_at_me();

  void render();

 protected:

  double r;

  double x;
  double y;

  double vx;
  double vy;

  /* How many collisions occurred in the recent past. If this number
     is too large, the object is assumed to be stuck somewhere and
     vanishes. */
  int ccount;

 private:
  bool triggers;
  bool gravity;

  double dr; // delta_radius

};

#endif
